@@ -4,69 +4,65 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2006 Torus Knot Software Ltd
-Also see acknowledgements in Readme.html
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU Lesser General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
 
-You should have received a copy of the GNU Lesser General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place - Suite 330, Boston, MA 02111-1307, USA, or go to
-http://www.gnu.org/copyleft/lesser.txt.
-
-You may alternatively use this source under the terms of a specific version of
-the OGRE Unrestricted License provided you have obtained such a license from
-Torus Knot Software Ltd.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
 
 #include "OgreMaterial.h"
-
-#include "OgreSceneManagerEnumerator.h"
-#include "OgreMaterialManager.h"
-#include "OgreIteratorWrappers.h"
-#include "OgreTechnique.h"
-#include "OgreLogManager.h"
-#include "OgreException.h"
-#include "OgreStringConverter.h"
+#include "OgreLodStrategyManager.h"
+#include "OgreLodStrategy.h"
 
 namespace Ogre {
 
     //-----------------------------------------------------------------------
-	Material::Material(ResourceManager* creator, const String& name, ResourceHandle handle,
-		const String& group, bool isManual, ManualResourceLoader* loader)
-		:Resource(creator, name, handle, group, isManual, loader),
+    Material::Material(ResourceManager* creator, const String& name, ResourceHandle handle,
+        const String& group, bool isManual, ManualResourceLoader* loader)
+        :Resource(creator, name, handle, group, isManual, loader),
          mReceiveShadows(true),
          mTransparencyCastsShadows(false),
          mCompilationRequired(true)
     {
-		// Override isManual, not applicable for Material (we always want to call loadImpl)
-		if(isManual)
-		{
-			mIsManual = false;
-			LogManager::getSingleton().logMessage("Material " + name + 
-				" was requested with isManual=true, but this is not applicable " 
-				"for materials; the flag has been reset to false");
-		}
+        // Override isManual, not applicable for Material (we always want to call loadImpl)
+        if(isManual)
+        {
+            mIsManual = false;
+            LogManager::getSingleton().logMessage("Material " + name + 
+                " was requested with isManual=true, but this is not applicable " 
+                "for materials; the flag has been reset to false");
+        }
 
-		mLodDistances.push_back(0.0f);
+        // Initialise to default strategy
+        mLodStrategy = LodStrategyManager::getSingleton().getDefaultStrategy();
 
-		applyDefaults();
+        mLodValues.push_back(0.0f);
 
-		/* For consistency with StringInterface, but we don't add any parameters here
-		That's because the Resource implementation of StringInterface is to
-		list all the options that need to be set before loading, of which 
-		we have none as such. Full details can be set through scripts.
-		*/ 
-		createParamDictionary("Material");
+        applyDefaults();
+
+        /* For consistency with StringInterface, but we don't add any parameters here
+        That's because the Resource implementation of StringInterface is to
+        list all the options that need to be set before loading, of which 
+        we have none as such. Full details can be set through scripts.
+        */ 
+        createParamDictionary("Material");
     }
     //-----------------------------------------------------------------------
     Material::~Material()
@@ -79,18 +75,18 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     Material& Material::operator=(const Material& rhs)
     {
-	    mName = rhs.mName;
-		mGroup = rhs.mGroup;
-		mCreator = rhs.mCreator;
-		mIsManual = rhs.mIsManual;
-		mLoader = rhs.mLoader;
-	    mHandle = rhs.mHandle;
+        mName = rhs.mName;
+        mGroup = rhs.mGroup;
+        mCreator = rhs.mCreator;
+        mIsManual = rhs.mIsManual;
+        mLoader = rhs.mLoader;
+        mHandle = rhs.mHandle;
         mSize = rhs.mSize;
         mReceiveShadows = rhs.mReceiveShadows;
         mTransparencyCastsShadows = rhs.mTransparencyCastsShadows;
 
-        mLoadingState = rhs.mLoadingState;
-		mIsBackgroundLoaded = rhs.mIsBackgroundLoaded;
+        mLoadingState.store(rhs.mLoadingState.load());
+        mIsBackgroundLoaded = rhs.mIsBackgroundLoaded;
 
         // Copy Techniques
         this->removeAllTechniques();
@@ -102,25 +98,27 @@ namespace Ogre {
             *t = *(*i);
             if ((*i)->isSupported())
             {
-				insertSupportedTechnique(t);
+                insertSupportedTechnique(t);
             }
         }
 
-		// Also copy LOD information
-		mLodDistances = rhs.mLodDistances;
+        // Also copy LOD information
+        mUserLodValues = rhs.mUserLodValues;
+        mLodValues = rhs.mLodValues;
+        mLodStrategy = rhs.mLodStrategy;
         mCompilationRequired = rhs.mCompilationRequired;
         // illumination passes are not compiled right away so
         // mIsLoaded state should still be the same as the original material
         assert(isLoaded() == rhs.isLoaded());
 
-	    return *this;
+        return *this;
     }
 
 
     //-----------------------------------------------------------------------
     void Material::prepareImpl(void)
     {
-		// compile if required
+        // compile if required
         if (mCompilationRequired)
             compile();
 
@@ -168,38 +166,51 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    MaterialPtr Material::clone(const String& newName, bool changeGroup, 
-		const String& newGroup) const
+    size_t Material::calculateSize(void) const
     {
-		MaterialPtr newMat;
-		if (changeGroup)
-		{
-			newMat = MaterialManager::getSingleton().create(newName, newGroup);
-		}
-		else
-		{
-			newMat = MaterialManager::getSingleton().create(newName, mGroup);
-		}
-        
+        size_t memSize = 0;
+
+        // Tally up techniques
+        Techniques::const_iterator i, iend;
+        iend = mTechniques.end();
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
+            memSize += (*i)->calculateSize();
+        }
+
+        memSize += sizeof(bool) * 3;
+        memSize += mUnsupportedReasons.size() * sizeof(char);
+        memSize += sizeof(LodStrategy);
+
+        memSize += Resource::calculateSize();
+
+        return memSize;
+    }
+    //-----------------------------------------------------------------------
+    MaterialPtr Material::clone(const String& newName, bool changeGroup, 
+        const String& newGroup) const
+    {
+        MaterialPtr newMat =
+            MaterialManager::getSingleton().create(newName, changeGroup ? newGroup : mGroup);
+
+        if(!newMat) // interception by collision handler
+            return newMat;
 
         // Keep handle (see below, copy overrides everything)
         ResourceHandle newHandle = newMat->getHandle();
         // Assign values from this
         *newMat = *this;
-		// Restore new group if required, will have been overridden by operator
-		if (changeGroup)
-		{
-			newMat->mGroup = newGroup;
-		}
-		
+        // Restore new group if required, will have been overridden by operator
+        if (changeGroup)
+        {
+            newMat->mGroup = newGroup;
+        }
+        
         // Correct the name & handle, they get copied too
         newMat->mName = newName;
         newMat->mHandle = newHandle;
 
         return newMat;
-
-
-
     }
     //-----------------------------------------------------------------------
     void Material::copyDetailsTo(MaterialPtr& mat) const
@@ -208,39 +219,39 @@ namespace Ogre {
         ResourceHandle savedHandle = mat->mHandle;
         String savedName = mat->mName;
         String savedGroup = mat->mGroup;
-		ManualResourceLoader* savedLoader = mat->mLoader;
-		bool savedManual = mat->mIsManual;
+        ManualResourceLoader* savedLoader = mat->mLoader;
+        bool savedManual = mat->mIsManual;
         // Assign values from this
         *mat = *this;
         // Correct the name & handle, they get copied too
         mat->mName = savedName;
         mat->mHandle = savedHandle;
         mat->mGroup = savedGroup;
-		mat->mIsManual = savedManual;
-		mat->mLoader = savedLoader;
+        mat->mIsManual = savedManual;
+        mat->mLoader = savedLoader;
 
     }
     //-----------------------------------------------------------------------
     void Material::applyDefaults(void)
     {
-		MaterialPtr defaults = MaterialManager::getSingleton().getDefaultSettings();
+        MaterialPtr defaults = MaterialManager::getSingleton().getDefaultSettings();
 
-		if (!defaults.isNull())
-		{
+        if (defaults)
+        {
             // save name & handle
             String savedName = mName;
             String savedGroup = mGroup;
             ResourceHandle savedHandle = mHandle;
-			ManualResourceLoader *savedLoader = mLoader;
-			bool savedManual = mIsManual;
-			*this = *defaults;
+            ManualResourceLoader *savedLoader = mLoader;
+            bool savedManual = mIsManual;
+            *this = *defaults;
             // restore name & handle
             mName = savedName;
             mHandle = savedHandle;
             mGroup = savedGroup;
-			mLoader = savedLoader;
-			mIsManual = savedManual;
-		}
+            mLoader = savedLoader;
+            mIsManual = savedManual;
+        }
         mCompilationRequired = true;
 
     }
@@ -253,16 +264,16 @@ namespace Ogre {
         return t;
     }
     //-----------------------------------------------------------------------
-    Technique* Material::getTechnique(unsigned short index)
+    Technique* Material::getTechnique(unsigned short index) const
     {
         assert (index < mTechniques.size() && "Index out of bounds.");
         return mTechniques[index];
     }
     //-----------------------------------------------------------------------
-    Technique* Material::getTechnique(const String& name)
+    Technique* Material::getTechnique(const String& name) const
     {
-        Techniques::iterator i    = mTechniques.begin();
-        Techniques::iterator iend = mTechniques.end();
+        Techniques::const_iterator i    = mTechniques.begin();
+        Techniques::const_iterator iend = mTechniques.end();
         Technique* foundTechnique = 0;
 
         // iterate through techniques to find a match
@@ -278,71 +289,71 @@ namespace Ogre {
 
         return foundTechnique;
     }
-    //-----------------------------------------------------------------------	
+    //-----------------------------------------------------------------------   
     unsigned short Material::getNumTechniques(void) const
     {
-		return static_cast<unsigned short>(mTechniques.size());
+        return static_cast<unsigned short>(mTechniques.size());
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     Technique* Material::getSupportedTechnique(unsigned short index)
     {
         assert (index < mSupportedTechniques.size() && "Index out of bounds.");
         return mSupportedTechniques[index];
     }
-    //-----------------------------------------------------------------------	
+    //-----------------------------------------------------------------------   
     unsigned short Material::getNumSupportedTechniques(void) const
     {
-		return static_cast<unsigned short>(mSupportedTechniques.size());
+        return static_cast<unsigned short>(mSupportedTechniques.size());
     }
-	//-----------------------------------------------------------------------
-	unsigned short Material::getNumLodLevels(unsigned short schemeIndex) const
-	{
-		// Safety check - empty list?
-		if (mBestTechniquesBySchemeList.empty())
-			return 0;
+    //-----------------------------------------------------------------------
+    unsigned short Material::getNumLodLevels(unsigned short schemeIndex) const
+    {
+        // Safety check - empty list?
+        if (mBestTechniquesBySchemeList.empty())
+            return 0;
 
-		BestTechniquesBySchemeList::const_iterator i = 
-			mBestTechniquesBySchemeList.find(schemeIndex);
-		if (i == mBestTechniquesBySchemeList.end())
-		{
-			// get the first item, will be 0 (the default) if default
-			// scheme techniques exist, otherwise the earliest defined
-			i = mBestTechniquesBySchemeList.begin();
-		}
+        BestTechniquesBySchemeList::const_iterator i = 
+            mBestTechniquesBySchemeList.find(schemeIndex);
+        if (i == mBestTechniquesBySchemeList.end())
+        {
+            // get the first item, will be 0 (the default) if default
+            // scheme techniques exist, otherwise the earliest defined
+            i = mBestTechniquesBySchemeList.begin();
+        }
 
-		return static_cast<unsigned short>(i->second->size());
-	}
-	//-----------------------------------------------------------------------
-	unsigned short Material::getNumLodLevels(const String& schemeName) const
-	{
-		return getNumLodLevels(
-			MaterialManager::getSingleton()._getSchemeIndex(schemeName));
-	}
-	//-----------------------------------------------------------------------
-	void Material::insertSupportedTechnique(Technique* t)
-	{
-		mSupportedTechniques.push_back(t);
-		// get scheme
-		unsigned short schemeIndex = t->_getSchemeIndex();
-		BestTechniquesBySchemeList::iterator i =
-			mBestTechniquesBySchemeList.find(schemeIndex);
-		LodTechniques* lodtechs = 0;
-		if (i == mBestTechniquesBySchemeList.end())
-		{
-			lodtechs = OGRE_NEW_T(LodTechniques, MEMCATEGORY_RESOURCE);
-			mBestTechniquesBySchemeList[schemeIndex] = lodtechs;
-		}
-		else
-		{
-			lodtechs = i->second;
-		}
+        return static_cast<unsigned short>(i->second->size());
+    }
+    //-----------------------------------------------------------------------
+    unsigned short Material::getNumLodLevels(const String& schemeName) const
+    {
+        return getNumLodLevels(
+            MaterialManager::getSingleton()._getSchemeIndex(schemeName));
+    }
+    //-----------------------------------------------------------------------
+    void Material::insertSupportedTechnique(Technique* t)
+    {
+        mSupportedTechniques.push_back(t);
+        // get scheme
+        unsigned short schemeIndex = t->_getSchemeIndex();
+        BestTechniquesBySchemeList::iterator i =
+            mBestTechniquesBySchemeList.find(schemeIndex);
+        LodTechniques* lodtechs = 0;
+        if (i == mBestTechniquesBySchemeList.end())
+        {
+            lodtechs = OGRE_NEW_T(LodTechniques, MEMCATEGORY_RESOURCE);
+            mBestTechniquesBySchemeList[schemeIndex] = lodtechs;
+        }
+        else
+        {
+            lodtechs = i->second;
+        }
 
-		// Insert won't replace if supported technique for this scheme/lod is
-		// already there, which is what we want
-		lodtechs->insert(LodTechniques::value_type(t->getLodIndex(), t));
+        // Insert won't replace if supported technique for this scheme/lod is
+        // already there, which is what we want
+        lodtechs->emplace(t->getLodIndex(), t);
 
-	}
-	//-----------------------------------------------------------------------------
+    }
+    //-----------------------------------------------------------------------------
     Technique* Material::getBestTechnique(unsigned short lodIndex, const Renderable* rend)
     {
         if (mSupportedTechniques.empty())
@@ -351,56 +362,56 @@ namespace Ogre {
         }
         else
         {
-			Technique* ret = 0;
-			MaterialManager& matMgr = MaterialManager::getSingleton();
-			// get scheme
-			BestTechniquesBySchemeList::iterator si = 
-				mBestTechniquesBySchemeList.find(matMgr._getActiveSchemeIndex());
-			// scheme not found?
-			if (si == mBestTechniquesBySchemeList.end())
-			{
-				// listener specified alternative technique available?
-				ret = matMgr._arbitrateMissingTechniqueForActiveScheme(this, lodIndex, rend);
-				if (ret)
-					return ret;
+            Technique* ret = 0;
+            MaterialManager& matMgr = MaterialManager::getSingleton();
+            // get scheme
+            BestTechniquesBySchemeList::iterator si = 
+                mBestTechniquesBySchemeList.find(matMgr._getActiveSchemeIndex());
+            // scheme not found?
+            if (si == mBestTechniquesBySchemeList.end())
+            {
+                // listener specified alternative technique available?
+                ret = matMgr._arbitrateMissingTechniqueForActiveScheme(this, lodIndex, rend);
+                if (ret)
+                    return ret;
 
-				// Nope, use default
-				// get the first item, will be 0 (the default) if default
-				// scheme techniques exist, otherwise the earliest defined
-				si = mBestTechniquesBySchemeList.begin();
-			}
+                // Nope, use default
+                // get the first item, will be 0 (the default) if default
+                // scheme techniques exist, otherwise the earliest defined
+                si = mBestTechniquesBySchemeList.begin();
+            }
 
-			// get LOD
-			LodTechniques::iterator li = si->second->find(lodIndex);
-			// LOD not found? 
-			if (li == si->second->end())
-			{
-				// Use the next LOD level up
-				for (LodTechniques::reverse_iterator rli = si->second->rbegin(); 
-					rli != si->second->rend(); ++rli)
-				{
-					if (rli->second->getLodIndex() < lodIndex)
-					{
-						ret = rli->second;
-						break;
-					}
+            // get LOD
+            LodTechniques::iterator li = si->second->find(lodIndex);
+            // LOD not found? 
+            if (li == si->second->end())
+            {
+                // Use the next LOD level up
+                for (LodTechniques::reverse_iterator rli = si->second->rbegin(); 
+                    rli != si->second->rend(); ++rli)
+                {
+                    if (rli->second->getLodIndex() < lodIndex)
+                    {
+                        ret = rli->second;
+                        break;
+                    }
 
-				}
-				if (!ret)
-				{
-					// shouldn't ever hit this really, unless user defines no LOD 0
-					// pick the first LOD we have (must be at least one to have a scheme entry)
-					ret = si->second->begin()->second;
-				}
+                }
+                if (!ret)
+                {
+                    // shouldn't ever hit this really, unless user defines no LOD 0
+                    // pick the first LOD we have (must be at least one to have a scheme entry)
+                    ret = si->second->begin()->second;
+                }
 
-			}
-			else
-			{
-				// LOD found
-				ret = li->second;
-			}
+            }
+            else
+            {
+                // LOD found
+                ret = li->second;
+            }
 
-			return ret;
+            return ret;
 
         }
     }
@@ -412,7 +423,7 @@ namespace Ogre {
         OGRE_DELETE(*i);
         mTechniques.erase(i);
         mSupportedTechniques.clear();
-		clearBestTechniqueList();
+        clearBestTechniqueList();
         mCompilationRequired = true;
     }
     //-----------------------------------------------------------------------
@@ -441,47 +452,47 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     bool Material::isTransparent(void) const
-	{
-		// Check each technique
-		Techniques::const_iterator i, iend;
-		iend = mTechniques.end();
-		for (i = mTechniques.begin(); i != iend; ++i)
-		{
-			if ( (*i)->isTransparent() )
-				return true;
-		}
-		return false;
-	}
+    {
+        // Check each technique
+        Techniques::const_iterator i, iend;
+        iend = mTechniques.end();
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
+            if ( (*i)->isTransparent() )
+                return true;
+        }
+        return false;
+    }
     //-----------------------------------------------------------------------
     void Material::compile(bool autoManageTextureUnits)
     {
         // Compile each technique, then add it to the list of supported techniques
         mSupportedTechniques.clear();
-		clearBestTechniqueList();
-		mUnsupportedReasons.clear();
+        clearBestTechniqueList();
+        mUnsupportedReasons.clear();
 
 
         Techniques::iterator i, iend;
         iend = mTechniques.end();
-		size_t techNo = 0;
+        size_t techNo = 0;
         for (i = mTechniques.begin(); i != iend; ++i, ++techNo)
         {
             String compileMessages = (*i)->_compile(autoManageTextureUnits);
             if ( (*i)->isSupported() )
             {
-				insertSupportedTechnique(*i);
+                insertSupportedTechnique(*i);
+            }
+            else
+            {
+                // Log informational
+                StringStream str;
+                str << "Material " << mName << " Technique " << techNo;
+                if (!(*i)->getName().empty())
+                    str << "(" << (*i)->getName() << ")";
+                str << " is not supported. " << compileMessages;
+                LogManager::getSingleton().logMessage(str.str(), LML_TRIVIAL);
+                mUnsupportedReasons += compileMessages;
             }
-			else
-			{
-				// Log informational
-				StringUtil::StrStreamType str;
-				str << "Material " << mName << " Technique " << techNo;
-				if (!(*i)->getName().empty())
-					str << "(" << (*i)->getName() << ")";
-				str << " is not supported. " << compileMessages;
-				LogManager::getSingleton().logMessage(str.str(), LML_TRIVIAL);
-				mUnsupportedReasons += compileMessages;
-			}
         }
 
         mCompilationRequired = false;
@@ -489,21 +500,21 @@ namespace Ogre {
         // Did we find any?
         if (mSupportedTechniques.empty())
         {
-			LogManager::getSingleton().stream()
-				<< "WARNING: material " << mName << " has no supportable "
-				<< "Techniques and will be blank. Explanation: \n" << mUnsupportedReasons;
+            LogManager::getSingleton().stream(LML_WARNING)
+                << "Warning: material " << mName << " has no supportable "
+                << "Techniques and will be blank. Explanation: \n" << mUnsupportedReasons;
         }
     }
-	//-----------------------------------------------------------------------
-	void Material::clearBestTechniqueList(void)
-	{
-		for (BestTechniquesBySchemeList::iterator i = mBestTechniquesBySchemeList.begin();
-			i != mBestTechniquesBySchemeList.end(); ++i)
-		{
-			OGRE_DELETE_T(i->second, LodTechniques, MEMCATEGORY_RESOURCE);
-		}
-		mBestTechniquesBySchemeList.clear();
-	}
+    //-----------------------------------------------------------------------
+    void Material::clearBestTechniqueList(void)
+    {
+        for (BestTechniquesBySchemeList::iterator i = mBestTechniquesBySchemeList.begin();
+            i != mBestTechniquesBySchemeList.end(); ++i)
+        {
+            OGRE_DELETE_T(i->second, LodTechniques, MEMCATEGORY_RESOURCE);
+        }
+        mBestTechniquesBySchemeList.clear();
+    }
     //-----------------------------------------------------------------------
     void Material::setPointSize(Real ps)
     {
@@ -516,23 +527,22 @@ namespace Ogre {
 
     }
     //-----------------------------------------------------------------------
-    void Material::setAmbient(Real red, Real green, Real blue)
+    void Material::setAmbient(float red, float green, float blue)
+    {
+        setAmbient(ColourValue(red, green, blue));
+    }
+    //-----------------------------------------------------------------------
+    void Material::setAmbient(const ColourValue& ambient)
     {
         Techniques::iterator i, iend;
         iend = mTechniques.end();
         for (i = mTechniques.begin(); i != iend; ++i)
         {
-            (*i)->setAmbient(red, green, blue);
+            (*i)->setAmbient(ambient);
         }
-
     }
     //-----------------------------------------------------------------------
-    void Material::setAmbient(const ColourValue& ambient)
-    {
-        setAmbient(ambient.r, ambient.g, ambient.b);
-    }
-    //-----------------------------------------------------------------------
-    void Material::setDiffuse(Real red, Real green, Real blue, Real alpha)
+    void Material::setDiffuse(float red, float green, float blue, float alpha)
     {
         Techniques::iterator i, iend;
         iend = mTechniques.end();
@@ -547,7 +557,7 @@ namespace Ogre {
         setDiffuse(diffuse.r, diffuse.g, diffuse.b, diffuse.a);
     }
     //-----------------------------------------------------------------------
-    void Material::setSpecular(Real red, Real green, Real blue, Real alpha)
+    void Material::setSpecular(float red, float green, float blue, float alpha)
     {
         Techniques::iterator i, iend;
         iend = mTechniques.end();
@@ -572,21 +582,21 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    void Material::setSelfIllumination(Real red, Real green, Real blue)
+    void Material::setSelfIllumination(float red, float green, float blue)
+    {
+        setSelfIllumination(ColourValue(red, green, blue));   
+    }
+    //-----------------------------------------------------------------------
+    void Material::setSelfIllumination(const ColourValue& selfIllum)
     {
         Techniques::iterator i, iend;
         iend = mTechniques.end();
         for (i = mTechniques.begin(); i != iend; ++i)
         {
-            (*i)->setSelfIllumination(red, green, blue);
+            (*i)->setSelfIllumination(selfIllum);
         }
     }
     //-----------------------------------------------------------------------
-    void Material::setSelfIllumination(const ColourValue& selfIllum)
-    {
-        setSelfIllumination(selfIllum.r, selfIllum.g, selfIllum.b);
-    }
-    //-----------------------------------------------------------------------
     void Material::setDepthCheckEnabled(bool enabled)
     {
         Techniques::iterator i, iend;
@@ -617,7 +627,7 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-	void Material::setColourWriteEnabled(bool enabled)
+    void Material::setColourWriteEnabled(bool enabled)
     {
         Techniques::iterator i, iend;
         iend = mTechniques.end();
@@ -627,6 +637,16 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
+    void Material::setColourWriteEnabled(bool red, bool green, bool blue, bool alpha)
+    {
+        Techniques::iterator i, iend;
+        iend = mTechniques.end();
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
+            (*i)->setColourWriteEnabled(red, green, blue, alpha);
+        }
+    }
+    //-----------------------------------------------------------------------
     void Material::setCullingMode( CullingMode mode )
     {
         Techniques::iterator i, iend;
@@ -740,79 +760,90 @@ namespace Ogre {
     }
     // --------------------------------------------------------------------
     void Material::setSeparateSceneBlending( const SceneBlendFactor sourceFactor, const SceneBlendFactor destFactor, const SceneBlendFactor sourceFactorAlpha, const SceneBlendFactor destFactorAlpha)
-	{
+    {
         Techniques::iterator i, iend;
         iend = mTechniques.end();
         for (i = mTechniques.begin(); i != iend; ++i)
         {
             (*i)->setSeparateSceneBlending(sourceFactor, destFactor, sourceFactorAlpha, destFactorAlpha);
         }
-	}
+    }
     // --------------------------------------------------------------------
     void Material::_notifyNeedsRecompile(void)
     {
         mCompilationRequired = true;
         // Also need to unload to ensure we loaded any new items
-		if (isLoaded()) // needed to stop this being called in 'loading' state
-			unload();
+        if (isLoaded()) // needed to stop this being called in 'loading' state
+            unload();
     }
     // --------------------------------------------------------------------
-    void Material::setLodLevels(const LodDistanceList& lodDistances)
+    void Material::setLodLevels(const LodValueList& lodValues)
     {
         // Square the distances for the internal list
-		LodDistanceList::const_iterator i, iend;
-		iend = lodDistances.end();
-		// First, clear and add single zero entry
-		mLodDistances.clear();
-		mLodDistances.push_back(0.0f);
-		for (i = lodDistances.begin(); i != iend; ++i)
-		{
-			mLodDistances.push_back((*i) * (*i));
-		}
-		
+        LodValueList::const_iterator i, iend;
+        iend = lodValues.end();
+        // First, clear and add single zero entry
+        mLodValues.clear();
+        mUserLodValues.clear();
+        mUserLodValues.push_back(0);
+        if (mLodStrategy)
+            mLodValues.push_back(mLodStrategy->getBaseValue());
+        for (i = lodValues.begin(); i != iend; ++i)
+        {
+            mUserLodValues.push_back(*i);
+            if (mLodStrategy)
+                mLodValues.push_back(mLodStrategy->transformUserValue(*i));
+        }
+        
     }
     // --------------------------------------------------------------------
-    unsigned short Material::getLodIndex(Real d) const
+    ushort Material::getLodIndex(Real value) const
     {
-        return getLodIndexSquaredDepth(d * d);
+        return mLodStrategy->getIndex(value, mLodValues);
     }
     // --------------------------------------------------------------------
-    unsigned short Material::getLodIndexSquaredDepth(Real squaredDistance) const
+    Material::LodValueIterator Material::getLodValueIterator(void) const
     {
-		LodDistanceList::const_iterator i, iend;
-		iend = mLodDistances.end();
-		unsigned short index = 0;
-		for (i = mLodDistances.begin(); i != iend; ++i, ++index)
-		{
-			if (*i > squaredDistance)
-			{
-				return index - 1;
-			}
-		}
-
-		// If we fall all the way through, use the highest value
-		return static_cast<ushort>(mLodDistances.size() - 1);
+        return LodValueIterator(mLodValues.begin(), mLodValues.end());
     }
     // --------------------------------------------------------------------
-    Material::LodDistanceIterator Material::getLodDistanceIterator(void) const
+    Material::LodValueIterator Material::getUserLodValueIterator(void) const
     {
-        return LodDistanceIterator(mLodDistances.begin(), mLodDistances.end());
+        return LodValueIterator(mUserLodValues.begin(), mUserLodValues.end());
     }
 
     //-----------------------------------------------------------------------
     bool Material::applyTextureAliases(const AliasTextureNamePairList& aliasList, const bool apply) const
     {
         // iterate through all techniques and apply texture aliases
-		Techniques::const_iterator i, iend;
-		iend = mTechniques.end();
+        Techniques::const_iterator i, iend;
+        iend = mTechniques.end();
         bool testResult = false;
 
-		for (i = mTechniques.begin(); i != iend; ++i)
-		{
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
             if ((*i)->applyTextureAliases(aliasList, apply))
                 testResult = true;
-		}
+        }
 
         return testResult;
     }
+    //---------------------------------------------------------------------
+    const LodStrategy *Material::getLodStrategy() const
+    {
+        return mLodStrategy;
+    }
+    //---------------------------------------------------------------------
+    void Material::setLodStrategy(LodStrategy *lodStrategy)
+    {
+        mLodStrategy = lodStrategy;
+
+        assert(mLodValues.size());
+        mLodValues[0] = mLodStrategy->getBaseValue();
+
+        // Re-transform all user LOD values (starting at index 1, no need to transform base value)
+        for (size_t i = 1; i < mUserLodValues.size(); ++i)
+            mLodValues[i] = mLodStrategy->transformUserValue(mUserLodValues[i]);
+    }
+    //---------------------------------------------------------------------
 }
