@@ -4,114 +4,71 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2006 Torus Knot Software Ltd
-Also see acknowledgements in Readme.html
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU Lesser General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
 
-You should have received a copy of the GNU Lesser General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place - Suite 330, Boston, MA 02111-1307, USA, or go to
-http://www.gnu.org/copyleft/lesser.txt.
-
-You may alternatively use this source under the terms of a specific version of
-the OGRE Unrestricted License provided you have obtained such a license from
-Torus Knot Software Ltd.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
-#include "OgreNode.h"
-
-#include "OgreException.h"
-#include "OgreMath.h"
-
-// Dependencies on render-related types due to ability to render node
-#include "OgreMaterialManager.h"
-#include "OgreMeshManager.h"
-#include "OgreMesh.h"
-#include "OgreSubMesh.h"
-#include "OgreCamera.h"
 
 namespace Ogre {
 
-    unsigned long Node::msNextGeneratedNameExt = 1;
-	Node::QueuedUpdates Node::msQueuedUpdates;
+    Node::QueuedUpdates Node::msQueuedUpdates;
     //-----------------------------------------------------------------------
-    Node::Node()
-		:mParent(0),
-		mNeedParentUpdate(false),
-		mNeedChildUpdate(false),
-		mParentNotified(false),
-        mQueuedForUpdate(false),
-		mOrientation(Quaternion::IDENTITY),
-		mPosition(Vector3::ZERO),
-		mScale(Vector3::UNIT_SCALE),
-        mInheritOrientation(true),
-		mInheritScale(true),
-		mDerivedOrientation(Quaternion::IDENTITY),
-		mDerivedPosition(Vector3::ZERO),
-		mDerivedScale(Vector3::UNIT_SCALE),
-		mInitialPosition(Vector3::ZERO),
-		mInitialOrientation(Quaternion::IDENTITY),
-		mInitialScale(Vector3::UNIT_SCALE),
-		mCachedTransformOutOfDate(true),
-		mListener(0)
-    {
-        // Generate a name
-		StringUtil::StrStreamType str;
-		str << "Unnamed_" << msNextGeneratedNameExt++;
-        mName = str.str();
-
-        needUpdate();
-
-    }
+    Node::Node() : Node(BLANKSTRING) {}
     //-----------------------------------------------------------------------
-	Node::Node(const String& name) : Renderable(),
-		mParent(0),
-		mNeedParentUpdate(false),
-		mNeedChildUpdate(false),
-		mParentNotified(false),
+    Node::Node(const String& name)
+        :mParent(0),
+        mName(name),
+        mNeedParentUpdate(false),
+        mNeedChildUpdate(false),
+        mParentNotified(false),
         mQueuedForUpdate(false),
-		mName(name),
-		mOrientation(Quaternion::IDENTITY),
-		mPosition(Vector3::ZERO),
-		mScale(Vector3::UNIT_SCALE),
         mInheritOrientation(true),
-		mInheritScale(true),
-		mDerivedOrientation(Quaternion::IDENTITY),
-		mDerivedPosition(Vector3::ZERO),
-		mDerivedScale(Vector3::UNIT_SCALE),
-		mInitialPosition(Vector3::ZERO),
-		mInitialOrientation(Quaternion::IDENTITY),
-		mInitialScale(Vector3::UNIT_SCALE),
-		mCachedTransformOutOfDate(true),
-		mListener(0)
-
+        mInheritScale(true),
+        mCachedTransformOutOfDate(true),
+        mOrientation(Quaternion::IDENTITY),
+        mPosition(Vector3::ZERO),
+        mScale(Vector3::UNIT_SCALE),
+        mDerivedOrientation(Quaternion::IDENTITY),
+        mDerivedPosition(Vector3::ZERO),
+        mDerivedScale(Vector3::UNIT_SCALE),
+        mInitialPosition(Vector3::ZERO),
+        mInitialOrientation(Quaternion::IDENTITY),
+        mInitialScale(Vector3::UNIT_SCALE),
+        mListener(0)
     {
-
         needUpdate();
-
     }
 
     //-----------------------------------------------------------------------
     Node::~Node()
     {
-		// Call listener (note, only called if there's something to do)
-		if (mListener)
-		{
-			mListener->nodeDestroyed(this);
-		}
+        // Call listener (note, only called if there's something to do)
+        if (mListener)
+        {
+            mListener->nodeDestroyed(this);
+        }
 
-		removeAllChildren();
-		if(mParent)
-			mParent->removeChild(this);
+        removeAllChildren();
+        if(mParent)
+            mParent->removeChild(this);
 
         if (mQueuedForUpdate)
         {
@@ -126,44 +83,70 @@ namespace Ogre {
                 msQueuedUpdates.pop_back();
             }
         }
-	}
-    //-----------------------------------------------------------------------
-    Node* Node::getParent(void) const
-    {
-        return mParent;
+
     }
 
     //-----------------------------------------------------------------------
     void Node::setParent(Node* parent)
     {
-		bool different = (parent != mParent);
+        bool different = (parent != mParent);
 
         mParent = parent;
         // Request update from parent
-		mParentNotified = false ;
+        mParentNotified = false ;
         needUpdate();
 
-		// Call listener (note, only called if there's something to do)
-		if (mListener && different)
-		{
-			if (mParent)
-				mListener->nodeAttached(this);
-			else
-				mListener->nodeDetached(this);
-		}
+        // Call listener (note, only called if there's something to do)
+        if (mListener && different)
+        {
+            if (mParent)
+                mListener->nodeAttached(this);
+            else
+                mListener->nodeDetached(this);
+        }
 
     }
 
     //-----------------------------------------------------------------------
-    const Matrix4& Node::_getFullTransform(void) const
+    const Affine3& Node::_getFullTransform(void) const
     {
         if (mCachedTransformOutOfDate)
         {
+#if OGRE_NODE_INHERIT_TRANSFORM
+            Affine3 tr;
+            tr.makeTransform(mPosition, mScale, mOrientation);
+
+            if(mParent == NULL)
+            {
+                mCachedTransform = tr;
+            }
+            else if(mInheritOrientation && mInheritScale) // everything is inherited
+            {
+                mCachedTransform = mParent->_getFullTransform() * tr;
+            }
+            else if(!mInheritOrientation && !mInheritScale) // only position is inherited
+            {
+                mCachedTransform = tr;
+                mCachedTransform.setTrans(tr.getTrans() + mParent->_getFullTransform().getTrans());
+            }
+            else // shear is inherited together with orientation, controlled by mInheritOrientation
+            {
+                const Affine3& parentTr = mParent->_getFullTransform();
+                Vector3 parentScale(
+                    parentTr.transformDirection(Vector3::UNIT_X).length(),
+                    parentTr.transformDirection(Vector3::UNIT_Y).length(),
+                    parentTr.transformDirection(Vector3::UNIT_Z).length());
+
+                assert(mInheritOrientation ^ mInheritScale);
+                mCachedTransform = (mInheritOrientation ? Affine3::getScale(1.0f / parentScale)  * parentTr : Affine3::getScale(parentScale)) * tr;
+            }
+#else
             // Use derived values
             mCachedTransform.makeTransform(
                 _getDerivedPosition(),
                 _getDerivedScale(),
                 _getDerivedOrientation());
+#endif
             mCachedTransformOutOfDate = false;
         }
         return mCachedTransform;
@@ -171,69 +154,67 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Node::_update(bool updateChildren, bool parentHasChanged)
     {
-		// always clear information about parent notification
-		mParentNotified = false ;
-
-        // Short circuit the off case
-        if (!updateChildren && !mNeedParentUpdate && !mNeedChildUpdate && !parentHasChanged )
-        {
-            return;
-        }
-
+        // always clear information about parent notification
+        mParentNotified = false;
 
         // See if we should process everyone
         if (mNeedParentUpdate || parentHasChanged)
         {
             // Update transforms from parent
             _updateFromParent();
-		}
-
-		if (mNeedChildUpdate || parentHasChanged)
-		{
-
-            ChildNodeMap::iterator it, itend;
-			itend = mChildren.end();
-            for (it = mChildren.begin(); it != itend; ++it)
-            {
-                Node* child = it->second;
-                child->_update(true, true);
-            }
-            mChildrenToUpdate.clear();
         }
-        else
+
+        if(updateChildren)
         {
-            // Just update selected children
-
-            ChildUpdateSet::iterator it, itend;
-			itend = mChildrenToUpdate.end();
-            for(it = mChildrenToUpdate.begin(); it != itend; ++it)
+            if (mNeedChildUpdate || parentHasChanged)
             {
-                Node* child = *it;
-                child->_update(true, false);
+                ChildNodeMap::iterator it, itend;
+                itend = mChildren.end();
+                for (it = mChildren.begin(); it != itend; ++it)
+                {
+                    Node* child = *it;
+                    child->_update(true, true);
+                }
+            }
+            else
+            {
+                // Just update selected children
+                ChildUpdateSet::iterator it, itend;
+                itend = mChildrenToUpdate.end();
+                for(it = mChildrenToUpdate.begin(); it != itend; ++it)
+                {
+                    Node* child = *it;
+                    child->_update(true, false);
+                }
+
             }
 
             mChildrenToUpdate.clear();
+            mNeedChildUpdate = false;
         }
-
-        mNeedChildUpdate = false;
-
     }
-	//-----------------------------------------------------------------------
-	void Node::_updateFromParent(void) const
-	{
-		updateFromParentImpl();
+    //-----------------------------------------------------------------------
+    void Node::_updateFromParent(void) const
+    {
+        updateFromParentImpl();
 
-		// Call listener (note, this method only called if there's something to do)
-		if (mListener)
-		{
-			mListener->nodeUpdated(this);
-		}
-	}
+        // Call listener (note, this method only called if there's something to do)
+        if (mListener)
+        {
+            mListener->nodeUpdated(this);
+        }
+    }
     //-----------------------------------------------------------------------
     void Node::updateFromParentImpl(void) const
     {
+        mCachedTransformOutOfDate = true;
+
         if (mParent)
         {
+#if OGRE_NODE_INHERIT_TRANSFORM
+            // Decompose full transform to position, orientation and scale, shear is lost here.
+            _getFullTransform().decomposition(mDerivedPosition, mDerivedScale, mDerivedOrientation);
+#else
             // Update orientation
             const Quaternion& parentOrientation = mParent->_getDerivedOrientation();
             if (mInheritOrientation)
@@ -241,9 +222,9 @@ namespace Ogre {
                 // Combine orientation with that of parent
                 mDerivedOrientation = parentOrientation * mOrientation;
             }
-			else
+            else
             {
-                // No inheritence
+                // No inheritance
                 mDerivedOrientation = mOrientation;
             }
 
@@ -257,7 +238,7 @@ namespace Ogre {
             }
             else
             {
-                // No inheritence
+                // No inheritance
                 mDerivedScale = mScale;
             }
 
@@ -266,6 +247,7 @@ namespace Ogre {
 
             // Add altered position vector to parents
             mDerivedPosition += mParent->_getDerivedPosition();
+#endif
         }
         else
         {
@@ -275,26 +257,26 @@ namespace Ogre {
             mDerivedScale = mScale;
         }
 
-		mCachedTransformOutOfDate = true;
-		mNeedParentUpdate = false;
+        mNeedParentUpdate = false;
 
     }
     //-----------------------------------------------------------------------
-    Node* Node::createChild(const Vector3& translate, const Quaternion& rotate)
+    Node* Node::createChild(const Vector3& inTranslate, const Quaternion& inRotate)
     {
         Node* newNode = createChildImpl();
-        newNode->translate(translate);
-        newNode->rotate(rotate);
+        newNode->setPosition(inTranslate);
+        newNode->setOrientation(inRotate);
         this->addChild(newNode);
 
         return newNode;
     }
     //-----------------------------------------------------------------------
-    Node* Node::createChild(const String& name, const Vector3& translate, const Quaternion& rotate)
+    Node* Node::createChild(const String& name, const Vector3& inTranslate, const Quaternion& inRotate)
     {
+        OgreAssert(!name.empty(), "name must not be empty");
         Node* newNode = createChildImpl(name);
-        newNode->translate(translate);
-        newNode->rotate(rotate);
+        newNode->setPosition(inTranslate);
+        newNode->setOrientation(inRotate);
         this->addChild(newNode);
 
         return newNode;
@@ -310,23 +292,16 @@ namespace Ogre {
                 "Node::addChild");
         }
 
-        mChildren.insert(ChildNodeMap::value_type(child->getName(), child));
+        mChildren.push_back(child);
         child->setParent(this);
 
     }
     //-----------------------------------------------------------------------
-    unsigned short Node::numChildren(void) const
-    {
-        return static_cast< unsigned short >( mChildren.size() );
-    }
-    //-----------------------------------------------------------------------
     Node* Node::getChild(unsigned short index) const
     {
         if( index < mChildren.size() )
         {
-            ChildNodeMap::const_iterator i = mChildren.begin();
-            while (index--) ++i;
-            return i->second;
+            return mChildren[index];
         }
         else
             return NULL;
@@ -334,16 +309,17 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     Node* Node::removeChild(unsigned short index)
     {
-        Node* ret;
         if (index < mChildren.size())
         {
             ChildNodeMap::iterator i = mChildren.begin();
-            while (index--) ++i;
-            ret = i->second;
+            i += index;
+            Node* ret = *i;
+
             // cancel any pending update
             cancelUpdate(ret);
 
-            mChildren.erase(i);
+            std::swap(*i, mChildren.back());
+            mChildren.pop_back();
             ret->setParent(NULL);
             return ret;
         }
@@ -361,39 +337,32 @@ namespace Ogre {
     {
         if (child)
         {
-            ChildNodeMap::iterator i = mChildren.find(child->getName());
-            // ensure it's our child
-            if (i != mChildren.end() && i->second == child)
+            ChildNodeMap::iterator i = std::find(mChildren.begin(), mChildren.end(), child);
+            if(i != mChildren.end() && *i == child)
             {
                 // cancel any pending update
                 cancelUpdate(child);
 
-                mChildren.erase(i);
+                std::swap(*i, mChildren.back());
+                mChildren.pop_back();
                 child->setParent(NULL);
             }
         }
         return child;
     }
-    //-----------------------------------------------------------------------
-    const Quaternion& Node::getOrientation() const
-    {
-        return mOrientation;
-    }
 
     //-----------------------------------------------------------------------
     void Node::setOrientation( const Quaternion & q )
     {
+        OgreAssertDbg(!q.isNaN(), "Invalid orientation supplied as parameter");
         mOrientation = q;
+        mOrientation.normalise();
         needUpdate();
     }
     //-----------------------------------------------------------------------
     void Node::setOrientation( Real w, Real x, Real y, Real z)
     {
-        mOrientation.w = w;
-        mOrientation.x = x;
-        mOrientation.y = y;
-        mOrientation.z = z;
-        needUpdate();
+        setOrientation(Quaternion(w, x, y, z));
     }
     //-----------------------------------------------------------------------
     void Node::resetOrientation(void)
@@ -405,6 +374,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Node::setPosition(const Vector3& pos)
     {
+        assert(!pos.isNaN() && "Invalid vector supplied as parameter");
         mPosition = pos;
         needUpdate();
     }
@@ -417,11 +387,6 @@ namespace Ogre {
         setPosition(v);
     }
 
-    //-----------------------------------------------------------------------
-    const Vector3 & Node::getPosition(void) const
-    {
-        return mPosition;
-    }
     //-----------------------------------------------------------------------
     Matrix3 Node::getLocalAxes(void) const
     {
@@ -446,19 +411,18 @@ namespace Ogre {
         case TS_LOCAL:
             // position is relative to parent so transform downwards
             mPosition += mOrientation * d;
-        	break;
+            break;
         case TS_WORLD:
             // position is relative to parent so transform upwards
             if (mParent)
             {
-                mPosition += (mParent->_getDerivedOrientation().Inverse() * d)
-                    / mParent->_getDerivedScale();
+                mPosition += mParent->convertWorldToLocalDirection(d, true);
             }
             else
             {
                 mPosition += d;
             }
-        	break;
+            break;
         case TS_PARENT:
             mPosition += d;
             break;
@@ -527,24 +491,45 @@ namespace Ogre {
             mOrientation = mOrientation * q;
             break;
         }
+
+        // Normalise quaternion to avoid drift
+        mOrientation.normalise();
+
         needUpdate();
     }
+
+    
+    //-----------------------------------------------------------------------
+    void Node::_setDerivedPosition( const Vector3& pos )
+    {
+        //find where the node would end up in parent's local space
+        if(mParent)
+            setPosition( mParent->convertWorldToLocalPosition( pos ) );
+    }
+    //-----------------------------------------------------------------------
+    void Node::_setDerivedOrientation( const Quaternion& q )
+    {
+        //find where the node would end up in parent's local space
+        if(mParent)
+            setOrientation( mParent->convertWorldToLocalOrientation( q ) );
+    }
+
     //-----------------------------------------------------------------------
     const Quaternion & Node::_getDerivedOrientation(void) const
     {
-		if (mNeedParentUpdate)
-		{
-        	_updateFromParent();
-		}
+        if (mNeedParentUpdate)
+        {
+            _updateFromParent();
+        }
         return mDerivedOrientation;
     }
     //-----------------------------------------------------------------------
     const Vector3 & Node::_getDerivedPosition(void) const
     {
-		if (mNeedParentUpdate)
-		{
-        	_updateFromParent();
-		}
+        if (mNeedParentUpdate)
+        {
+            _updateFromParent();
+        }
         return mDerivedPosition;
     }
     //-----------------------------------------------------------------------
@@ -557,36 +542,97 @@ namespace Ogre {
         return mDerivedScale;
     }
     //-----------------------------------------------------------------------
-    void Node::removeAllChildren(void)
+    Vector3 Node::convertWorldToLocalPosition( const Vector3 &worldPos )
     {
-		ChildNodeMap::iterator i, iend;
-		iend = mChildren.end();
-		for (i = mChildren.begin(); i != iend; ++i)
-		{
-			i->second->setParent(0);
-		}
-        mChildren.clear();
-		mChildrenToUpdate.clear();
+        if (mNeedParentUpdate)
+        {
+            _updateFromParent();
+        }
+#if OGRE_NODE_INHERIT_TRANSFORM
+        return _getFullTransform().inverse() * worldPos;
+#else
+        return mDerivedOrientation.Inverse() * (worldPos - mDerivedPosition) / mDerivedScale;
+#endif
     }
     //-----------------------------------------------------------------------
-    void Node::setScale(const Vector3& scale)
+    Vector3 Node::convertLocalToWorldPosition( const Vector3 &localPos )
     {
-        mScale = scale;
+        if (mNeedParentUpdate)
+        {
+            _updateFromParent();
+        }
+        return _getFullTransform() * localPos;
+    }
+    //-----------------------------------------------------------------------
+    Vector3 Node::convertWorldToLocalDirection( const Vector3 &worldDir, bool useScale )
+    {
+        if (mNeedParentUpdate)
+        {
+            _updateFromParent();
+        }
+
+        return useScale ? 
+#if OGRE_NODE_INHERIT_TRANSFORM
+            _getFullTransform().inverseAffine().transformDirectionAffine(worldDir) :
+            mDerivedOrientation.Inverse() * worldDir;
+#else
+            mDerivedOrientation.Inverse() * worldDir / mDerivedScale :
+            mDerivedOrientation.Inverse() * worldDir;
+#endif
+    }
+    //-----------------------------------------------------------------------
+    Vector3 Node::convertLocalToWorldDirection( const Vector3 &localDir, bool useScale )
+    {
+        if (mNeedParentUpdate)
+        {
+            _updateFromParent();
+        }
+        return useScale ? _getFullTransform().linear() * localDir : mDerivedOrientation * localDir;
+    }
+    //-----------------------------------------------------------------------
+    Quaternion Node::convertWorldToLocalOrientation( const Quaternion &worldOrientation )
+    {
+        if (mNeedParentUpdate)
+        {
+            _updateFromParent();
+        }
+        return mDerivedOrientation.Inverse() * worldOrientation;
+    }
+    //-----------------------------------------------------------------------
+    Quaternion Node::convertLocalToWorldOrientation( const Quaternion &localOrientation )
+    {
+        if (mNeedParentUpdate)
+        {
+            _updateFromParent();
+        }
+        return mDerivedOrientation * localOrientation;
+
+    }
+    //-----------------------------------------------------------------------
+    void Node::removeAllChildren(void)
+    {
+        ChildNodeMap::iterator i, iend;
+        iend = mChildren.end();
+        for (i = mChildren.begin(); i != iend; ++i)
+        {
+            (*i)->setParent(0);
+        }
+        mChildren.clear();
+        mChildrenToUpdate.clear();
+    }
+    //-----------------------------------------------------------------------
+    void Node::setScale(const Vector3& inScale)
+    {
+        assert(!inScale.isNaN() && "Invalid vector supplied as parameter");
+        mScale = inScale;
         needUpdate();
     }
     //-----------------------------------------------------------------------
     void Node::setScale(Real x, Real y, Real z)
     {
-        mScale.x = x;
-        mScale.y = y;
-        mScale.z = z;
-        needUpdate();
-    }
-    //-----------------------------------------------------------------------
-    const Vector3 & Node::getScale(void) const
-    {
-        return mScale;
+        setScale(Vector3(x, y, z));
     }
+
     //-----------------------------------------------------------------------
     void Node::setInheritOrientation(bool inherit)
     {
@@ -594,25 +640,15 @@ namespace Ogre {
         needUpdate();
     }
     //-----------------------------------------------------------------------
-    bool Node::getInheritOrientation(void) const
-    {
-        return mInheritOrientation;
-    }
-    //-----------------------------------------------------------------------
     void Node::setInheritScale(bool inherit)
     {
         mInheritScale = inherit;
         needUpdate();
     }
     //-----------------------------------------------------------------------
-    bool Node::getInheritScale(void) const
+    void Node::scale(const Vector3& inScale)
     {
-        return mInheritScale;
-    }
-    //-----------------------------------------------------------------------
-    void Node::scale(const Vector3& scale)
-    {
-        mScale = mScale * scale;
+        mScale = mScale * inScale;
         needUpdate();
 
     }
@@ -626,53 +662,6 @@ namespace Ogre {
 
     }
     //-----------------------------------------------------------------------
-    const String& Node::getName(void) const
-    {
-        return mName;
-    }
-    //-----------------------------------------------------------------------
-    const MaterialPtr& Node::getMaterial(void) const
-    {
-        if (mpMaterial.isNull())
-        {
-            mpMaterial = MaterialManager::getSingleton().getByName("Core/NodeMaterial");
-			if (mpMaterial.isNull())
-				OGRE_EXCEPT( Exception::ERR_ITEM_NOT_FOUND, "Could not find material Core/NodeMaterial",
-					"Node::getMaterial" );
-            mpMaterial->load();
-        }
-        return mpMaterial;
-
-    }
-    //-----------------------------------------------------------------------
-    void Node::getRenderOperation(RenderOperation& op)
-    {
-        static SubMesh* pSubMesh = 0;
-        if (!pSubMesh)
-        {
-            MeshPtr pMesh = MeshManager::getSingleton().load("axes.mesh",
-				ResourceGroupManager::BOOTSTRAP_RESOURCE_GROUP_NAME);
-            pSubMesh = pMesh->getSubMesh(0);
-        }
-        pSubMesh->_getRenderOperation(op);
-    }
-    //-----------------------------------------------------------------------
-    void Node::getWorldTransforms(Matrix4* xform) const
-    {
-        // Assumes up to date
-        *xform = this->_getFullTransform();
-    }
-    //-----------------------------------------------------------------------
-    const Quaternion& Node::getWorldOrientation(void) const
-    {
-        return _getDerivedOrientation();
-    }
-    //-----------------------------------------------------------------------
-    const Vector3& Node::getWorldPosition(void) const
-    {
-        return _getDerivedPosition();
-    }
-    //-----------------------------------------------------------------------
     void Node::setInitialState(void)
     {
         mInitialPosition = mPosition;
@@ -689,38 +678,31 @@ namespace Ogre {
         needUpdate();
     }
     //-----------------------------------------------------------------------
-    const Vector3& Node::getInitialPosition(void) const
-    {
-        return mInitialPosition;
-    }
-    //-----------------------------------------------------------------------
-    const Quaternion& Node::getInitialOrientation(void) const
-    {
-        return mInitialOrientation;
-
-    }
-    //-----------------------------------------------------------------------
-    const Vector3& Node::getInitialScale(void) const
-    {
-        return mInitialScale;
-    }
-    //-----------------------------------------------------------------------
+    struct NodeNameExists {
+        const String& name;
+        bool operator()(const Node* mo) {
+            return mo->getName() == name;
+        }
+    };
     Node* Node::getChild(const String& name) const
     {
-        ChildNodeMap::const_iterator i = mChildren.find(name);
+        NodeNameExists pred = {name};
+        ChildNodeMap::const_iterator i = std::find_if(mChildren.begin(), mChildren.end(), pred);
 
         if (i == mChildren.end())
         {
             OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Child node named " + name +
                 " does not exist.", "Node::getChild");
         }
-        return i->second;
 
+        return *i;
     }
     //-----------------------------------------------------------------------
     Node* Node::removeChild(const String& name)
     {
-        ChildNodeMap::iterator i = mChildren.find(name);
+        OgreAssert(!name.empty(), "name must not be empty");
+        NodeNameExists pred = {name};
+        ChildNodeMap::iterator i = std::find_if(mChildren.begin(), mChildren.end(), pred);
 
         if (i == mChildren.end())
         {
@@ -728,11 +710,12 @@ namespace Ogre {
                 " does not exist.", "Node::removeChild");
         }
 
-        Node* ret = i->second;
+        Node* ret = *i;
+
         // Cancel any pending update
         cancelUpdate(ret);
-
-        mChildren.erase(i);
+        std::swap(*i, mChildren.back());
+        mChildren.pop_back();
         ret->setParent(NULL);
 
         return ret;
@@ -744,11 +727,11 @@ namespace Ogre {
     {
         return ChildNodeIterator(mChildren.begin(), mChildren.end());
     }
-	//-----------------------------------------------------------------------
-	Node::ConstChildNodeIterator Node::getChildIterator(void) const
-	{
-		return ConstChildNodeIterator(mChildren.begin(), mChildren.end());
-	}
+    //-----------------------------------------------------------------------
+    Node::ConstChildNodeIterator Node::getChildIterator(void) const
+    {
+        return ConstChildNodeIterator(mChildren.begin(), mChildren.end());
+    }
     //-----------------------------------------------------------------------
     Real Node::getSquaredViewDepth(const Camera* cam) const
     {
@@ -762,14 +745,14 @@ namespace Ogre {
     {
 
         mNeedParentUpdate = true;
-		mNeedChildUpdate = true;
+        mNeedChildUpdate = true;
         mCachedTransformOutOfDate = true;
 
         // Make sure we're not root and parent hasn't been notified before
         if (mParent && (!mParentNotified || forceParentUpdate))
         {
             mParent->requestUpdate(this, forceParentUpdate);
-			mParentNotified = true ;
+            mParentNotified = true ;
         }
 
         // all children will be updated
@@ -787,10 +770,10 @@ namespace Ogre {
         mChildrenToUpdate.insert(child);
         // Request selective update of me, if we didn't do it before
         if (mParent && (!mParentNotified || forceParentUpdate))
-		{
+        {
             mParent->requestUpdate(this, forceParentUpdate);
-			mParentNotified = true ;
-		}
+            mParentNotified = true ;
+        }
 
     }
     //-----------------------------------------------------------------------
@@ -798,43 +781,112 @@ namespace Ogre {
     {
         mChildrenToUpdate.erase(child);
 
-        // Propogate this up if we're done
+        // Propagate this up if we're done
         if (mChildrenToUpdate.empty() && mParent && !mNeedChildUpdate)
         {
             mParent->cancelUpdate(this);
-			mParentNotified = false ;
+            mParentNotified = false ;
         }
     }
-	//-----------------------------------------------------------------------
-	void Node::queueNeedUpdate(Node* n)
-	{
+    //-----------------------------------------------------------------------
+    void Node::queueNeedUpdate(Node* n)
+    {
         // Don't queue the node more than once
         if (!n->mQueuedForUpdate)
         {
             n->mQueuedForUpdate = true;
-		    msQueuedUpdates.push_back(n);
+            msQueuedUpdates.push_back(n);
         }
-	}
-	//-----------------------------------------------------------------------
-	void Node::processQueuedUpdates(void)
-	{
-		for (QueuedUpdates::iterator i = msQueuedUpdates.begin();
-			i != msQueuedUpdates.end(); ++i)
-		{
-			// Update, and force parent update since chances are we've ended
-			// up with some mixed state in there due to re-entrancy
+    }
+    //-----------------------------------------------------------------------
+    void Node::processQueuedUpdates(void)
+    {
+        for (QueuedUpdates::iterator i = msQueuedUpdates.begin();
+            i != msQueuedUpdates.end(); ++i)
+        {
+            // Update, and force parent update since chances are we've ended
+            // up with some mixed state in there due to re-entrancy
             Node* n = *i;
             n->mQueuedForUpdate = false;
-			n->needUpdate(true);
-		}
-		msQueuedUpdates.clear();
-	}
+            n->needUpdate(true);
+        }
+        msQueuedUpdates.clear();
+    }
+    //---------------------------------------------------------------------
+    Node::DebugRenderable* Node::getDebugRenderable(Real scaling)
+    {
+        if (!mDebug)
+        {
+            mDebug.reset(new DebugRenderable(this));
+        }
+        mDebug->setScaling(scaling);
+        return mDebug.get();
+    }
+    //---------------------------------------------------------------------
     //-----------------------------------------------------------------------
-    const LightList& Node::getLights(void) const
+    Node::DebugRenderable::DebugRenderable(Node* parent)
+        : mParent(parent)
+    {
+        String matName = "Ogre/Debug/AxesMat";
+        mMat = MaterialManager::getSingleton().getByName(matName, ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);
+        if (!mMat)
+        {
+            mMat = MaterialManager::getSingleton().create(matName, ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);
+            Pass* p = mMat->getTechnique(0)->getPass(0);
+            p->setLightingEnabled(false);
+            p->setPolygonModeOverrideable(false);
+            p->setVertexColourTracking(TVC_AMBIENT);
+            p->setSceneBlending(SBT_TRANSPARENT_ALPHA);
+            p->setCullingMode(CULL_NONE);
+            p->setDepthWriteEnabled(false);
+            p->setDepthCheckEnabled(false);
+        }
+
+        String meshName = "Ogre/Debug/AxesMesh";
+        mMeshPtr = MeshManager::getSingleton().getByName(meshName, ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME);
+        if (!mMeshPtr->isLoaded())
+        {
+            mMeshPtr->load();
+            mMeshPtr->getSubMeshes()[0]->setMaterial(mMat);
+        }
+
+    }
+    //---------------------------------------------------------------------
+    Node::DebugRenderable::~DebugRenderable()
+    {
+    }
+    //-----------------------------------------------------------------------
+    const MaterialPtr& Node::DebugRenderable::getMaterial(void) const
+    {
+        return mMat;
+    }
+    //---------------------------------------------------------------------
+    void Node::DebugRenderable::getRenderOperation(RenderOperation& op)
+    {
+        return mMeshPtr->getSubMesh(0)->_getRenderOperation(op);
+    }
+    //-----------------------------------------------------------------------
+    void Node::DebugRenderable::getWorldTransforms(Matrix4* xform) const
+    {
+        // Assumes up to date
+        *xform = mParent->_getFullTransform();
+        if (!Math::RealEqual(mScaling, 1.0))
+        {
+            *xform = (*xform) * Affine3::getScale(mScaling, mScaling, mScaling);
+        }
+    }
+    //-----------------------------------------------------------------------
+    Real Node::DebugRenderable::getSquaredViewDepth(const Camera* cam) const
+    {
+        return mParent->getSquaredViewDepth(cam);
+    }
+    //-----------------------------------------------------------------------
+    const LightList& Node::DebugRenderable::getLights(void) const
     {
         // Nodes should not be lit by the scene, this will not get called
         static LightList ll;
         return ll;
     }
+
 }
 
