@@ -4,38 +4,31 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2006 Torus Knot Software Ltd
-Also see acknowledgements in Readme.html
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU Lesser General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
 
-You should have received a copy of the GNU Lesser General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place - Suite 330, Boston, MA 02111-1307, USA, or go to
-http://www.gnu.org/copyleft/lesser.txt.
-
-You may alternatively use this source under the terms of a specific version of
-the OGRE Unrestricted License provided you have obtained such a license from
-Torus Knot Software Ltd.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
-#include "OgreSkeleton.h"
-#include "OgreBone.h"
-#include "OgreAnimation.h"
 #include "OgreAnimationState.h"
-#include "OgreException.h"
-#include "OgreLogManager.h"
 #include "OgreSkeletonManager.h"
 #include "OgreSkeletonSerializer.h"
-#include "OgreStringConverter.h"
 // Just for logging
 #include "OgreAnimationTrack.h"
 #include "OgreKeyFrame.h"
@@ -44,14 +37,14 @@ Torus Knot Software Ltd.
 namespace Ogre {
 
     //---------------------------------------------------------------------
-	Skeleton::Skeleton()
-		: Resource(),
+    Skeleton::Skeleton()
+        : Resource(),
         mBlendState(ANIMBLEND_AVERAGE),
-		mNextAutoHandle(0),
-		mManualBonesDirty(false)
-	{
-	}
-	//---------------------------------------------------------------------
+        mNextAutoHandle(0),
+        mManualBonesDirty(false)
+    {
+    }
+    //---------------------------------------------------------------------
     Skeleton::Skeleton(ResourceManager* creator, const String& name, ResourceHandle handle,
         const String& group, bool isManual, ManualResourceLoader* loader) 
         : Resource(creator, name, handle, group, isManual, loader), 
@@ -74,23 +67,23 @@ namespace Ogre {
     void Skeleton::loadImpl(void)
     {
         SkeletonSerializer serializer;
-		LogManager::getSingleton().stream()
-			<< "Skeleton: Loading " << mName;
+        LogManager::getSingleton().stream()
+            << "Skeleton: Loading " << mName;
 
         DataStreamPtr stream = 
             ResourceGroupManager::getSingleton().openResource(
-				mName, mGroup, true, this);
+                mName, mGroup, this);
 
         serializer.importSkeleton(stream, this);
 
-		// Load any linked skeletons
-		LinkedSkeletonAnimSourceList::iterator i;
-		for (i = mLinkedSkeletonAnimSourceList.begin(); 
-			i != mLinkedSkeletonAnimSourceList.end(); ++i)
-		{
-			i->pSkeleton = SkeletonManager::getSingleton().load(
-				i->skeletonName, mGroup);
-		}
+        // Load any linked skeletons
+        LinkedSkeletonAnimSourceList::iterator i;
+        for (i = mLinkedSkeletonAnimSourceList.begin(); 
+            i != mLinkedSkeletonAnimSourceList.end(); ++i)
+        {
+            i->pSkeleton = static_pointer_cast<Skeleton>(
+                SkeletonManager::getSingleton().load(i->skeletonName, mGroup));
+        }
 
 
     }
@@ -105,7 +98,7 @@ namespace Ogre {
         }
         mBoneList.clear();
         mBoneListByName.clear();
-		mRootBones.clear();
+        mRootBones.clear();
         mManualBones.clear();
         mManualBonesDirty = false;
 
@@ -192,18 +185,15 @@ namespace Ogre {
         return ret;
     }
 
-
-
-    //---------------------------------------------------------------------
-    Bone* Skeleton::getRootBone(void) const
-    {
+    const Skeleton::BoneList& Skeleton::getRootBones() const {
         if (mRootBones.empty())
         {
             deriveRootBone();
         }
 
-        return mRootBones[0];
+        return mRootBones;
     }
+
     //---------------------------------------------------------------------
     void Skeleton::setAnimationState(const AnimationStateSet& animSet)
     {
@@ -216,37 +206,35 @@ namespace Ogre {
         // Reset bones
         reset();
 
-		Real weightFactor = 1.0f;
-		if (mBlendState == ANIMBLEND_AVERAGE)
-		{
-			// Derive total weights so we can rebalance if > 1.0f
-			Real totalWeights = 0.0f;
-			ConstEnabledAnimationStateIterator stateIt = 
-				animSet.getEnabledAnimationStateIterator();
-			while (stateIt.hasMoreElements())
-			{
-				const AnimationState* animState = stateIt.getNext();
-				// Make sure we have an anim to match implementation
-				const LinkedSkeletonAnimationSource* linked = 0;
-				if (_getAnimationImpl(animState->getAnimationName(), &linked))
-				{
-					totalWeights += animState->getWeight();
-				}
-			}
+        Real weightFactor = 1.0f;
+        if (mBlendState == ANIMBLEND_AVERAGE)
+        {
+            // Derive total weights so we can rebalance if > 1.0f
+            Real totalWeights = 0.0f;
+            EnabledAnimationStateList::const_iterator animIt;
+            for(animIt = animSet.getEnabledAnimationStates().begin(); animIt != animSet.getEnabledAnimationStates().end(); ++animIt)
+            {
+                const AnimationState* animState = *animIt;
+                // Make sure we have an anim to match implementation
+                const LinkedSkeletonAnimationSource* linked = 0;
+                if (_getAnimationImpl(animState->getAnimationName(), &linked))
+                {
+                    totalWeights += animState->getWeight();
+                }
+            }
 
-			// Allow < 1.0f, allows fade out of all anims if required 
-			if (totalWeights > 1.0f)
-			{
-				weightFactor = 1.0f / totalWeights;
-			}
-		}
+            // Allow < 1.0f, allows fade out of all anims if required 
+            if (totalWeights > 1.0f)
+            {
+                weightFactor = 1.0f / totalWeights;
+            }
+        }
 
         // Per enabled animation state
-        ConstEnabledAnimationStateIterator stateIt = 
-            animSet.getEnabledAnimationStateIterator();
-        while (stateIt.hasMoreElements())
+        EnabledAnimationStateList::const_iterator animIt;
+        for(animIt = animSet.getEnabledAnimationStates().begin(); animIt != animSet.getEnabledAnimationStates().end(); ++animIt)
         {
-            const AnimationState* animState = stateIt.getNext();
+            const AnimationState* animState = *animIt;
             const LinkedSkeletonAnimationSource* linked = 0;
             Animation* anim = _getAnimationImpl(animState->getAnimationName(), &linked);
             // tolerate state entries for animations we're not aware of
@@ -303,6 +291,7 @@ namespace Ogre {
         }
 
         Animation* ret = OGRE_NEW Animation(name, length);
+        ret->_notifyContainer(this);
 
         // Add to list
         mAnimationsList[name] = ret;
@@ -310,57 +299,62 @@ namespace Ogre {
         return ret;
 
     }
-	//---------------------------------------------------------------------
-	Animation* Skeleton::getAnimation(const String& name, 
-		const LinkedSkeletonAnimationSource** linker) const
-	{
-		Animation* ret = _getAnimationImpl(name, linker);
-		if (!ret)
-		{
-			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "No animation entry found named " + name, 
-				"Skeleton::getAnimation");
-		}
+    //---------------------------------------------------------------------
+    Animation* Skeleton::getAnimation(const String& name, 
+        const LinkedSkeletonAnimationSource** linker) const
+    {
+        Animation* ret = _getAnimationImpl(name, linker);
+        if (!ret)
+        {
+            OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "No animation entry found named " + name, 
+                "Skeleton::getAnimation");
+        }
 
-		return ret;
-	}
-	//---------------------------------------------------------------------
-	bool Skeleton::hasAnimation(const String& name)
-	{
-		return _getAnimationImpl(name) != 0;
-	}
+        return ret;
+    }
+    //---------------------------------------------------------------------
+    Animation* Skeleton::getAnimation(const String& name) const
+    {
+        return getAnimation(name, 0);
+    }
+    //---------------------------------------------------------------------
+    bool Skeleton::hasAnimation(const String& name) const
+    {
+        return _getAnimationImpl(name) != 0;
+    }
     //---------------------------------------------------------------------
     Animation* Skeleton::_getAnimationImpl(const String& name, 
-		const LinkedSkeletonAnimationSource** linker) const
+        const LinkedSkeletonAnimationSource** linker) const
     {
-		Animation* ret = 0;
+        Animation* ret = 0;
         AnimationList::const_iterator i = mAnimationsList.find(name);
 
         if (i == mAnimationsList.end())
         {
-			LinkedSkeletonAnimSourceList::const_iterator i;
-			for (i = mLinkedSkeletonAnimSourceList.begin(); 
-				i != mLinkedSkeletonAnimSourceList.end() && !ret; ++i)
-			{
-				if (!i->pSkeleton.isNull())
-				{
-					ret = i->pSkeleton->_getAnimationImpl(name);
-					if (ret && linker)
-					{
-						*linker = &(*i);
-					}
+            LinkedSkeletonAnimSourceList::const_iterator it;
+            for (it = mLinkedSkeletonAnimSourceList.begin(); 
+                it != mLinkedSkeletonAnimSourceList.end() && !ret; ++it)
+            {
+                if (it->pSkeleton)
+                {
+                    ret = it->pSkeleton->_getAnimationImpl(name);
+                    if (ret && linker)
+                    {
+                        *linker = &(*it);
+                    }
 
-				}
-			}
+                }
+            }
 
         }
-		else
-		{
-			if (linker)
-				*linker = 0;
-			ret = i->second;
-		}
+        else
+        {
+            if (linker)
+                *linker = 0;
+            ret = i->second;
+        }
 
-		return ret;
+        return ret;
 
     }
     //---------------------------------------------------------------------
@@ -393,71 +387,71 @@ namespace Ogre {
             animSet->createAnimationState(animName, 0.0, anim->getLength());
         }
 
-		// Also iterate over linked animation
-		LinkedSkeletonAnimSourceList::iterator li;
-		for (li = mLinkedSkeletonAnimSourceList.begin(); 
-			li != mLinkedSkeletonAnimSourceList.end(); ++li)
-		{
-			if (!li->pSkeleton.isNull())
-			{
-				li->pSkeleton->_refreshAnimationState(animSet);
-			}
-		}
+        // Also iterate over linked animation
+        LinkedSkeletonAnimSourceList::iterator li;
+        for (li = mLinkedSkeletonAnimSourceList.begin(); 
+            li != mLinkedSkeletonAnimSourceList.end(); ++li)
+        {
+            if (li->pSkeleton)
+            {
+                li->pSkeleton->_refreshAnimationState(animSet);
+            }
+        }
 
     }
-	//-----------------------------------------------------------------------
-	void Skeleton::_refreshAnimationState(AnimationStateSet* animSet)
-	{
-		// Merge in any new animations
-		AnimationList::iterator i;
-		for (i = mAnimationsList.begin(); i != mAnimationsList.end(); ++i)
-		{
-			Animation* anim = i->second;
-			// Create animation at time index 0, default params mean this has weight 1 and is disabled
-			const String& animName = anim->getName();
-			if (!animSet->hasAnimationState(animName))
-			{
-				animSet->createAnimationState(animName, 0.0, anim->getLength());
-			}
-			else
-			{
-				// Update length incase changed
-				AnimationState* animState = animSet->getAnimationState(animName);
-				animState->setLength(anim->getLength());
-				animState->setTimePosition(std::min(anim->getLength(), animState->getTimePosition()));
-			}
-		}
-		// Also iterate over linked animation
-		LinkedSkeletonAnimSourceList::iterator li;
-		for (li = mLinkedSkeletonAnimSourceList.begin(); 
-			li != mLinkedSkeletonAnimSourceList.end(); ++li)
-		{
-			if (!li->pSkeleton.isNull())
-			{
-				li->pSkeleton->_refreshAnimationState(animSet);
-			}
-		}
-	}
-	//-----------------------------------------------------------------------
-	void Skeleton::_notifyManualBonesDirty(void)
-	{
-		mManualBonesDirty = true;
-	}
-	//-----------------------------------------------------------------------
-	void Skeleton::_notifyManualBoneStateChange(Bone* bone)
-	{
-		if (bone->isManuallyControlled())
-			mManualBones.insert(bone);
-		else
-			mManualBones.erase(bone);
-	}
+    //-----------------------------------------------------------------------
+    void Skeleton::_refreshAnimationState(AnimationStateSet* animSet)
+    {
+        // Merge in any new animations
+        AnimationList::iterator i;
+        for (i = mAnimationsList.begin(); i != mAnimationsList.end(); ++i)
+        {
+            Animation* anim = i->second;
+            // Create animation at time index 0, default params mean this has weight 1 and is disabled
+            const String& animName = anim->getName();
+            if (!animSet->hasAnimationState(animName))
+            {
+                animSet->createAnimationState(animName, 0.0, anim->getLength());
+            }
+            else
+            {
+                // Update length incase changed
+                AnimationState* animState = animSet->getAnimationState(animName);
+                animState->setLength(anim->getLength());
+                animState->setTimePosition(std::min(anim->getLength(), animState->getTimePosition()));
+            }
+        }
+        // Also iterate over linked animation
+        LinkedSkeletonAnimSourceList::iterator li;
+        for (li = mLinkedSkeletonAnimSourceList.begin(); 
+            li != mLinkedSkeletonAnimSourceList.end(); ++li)
+        {
+            if (li->pSkeleton)
+            {
+                li->pSkeleton->_refreshAnimationState(animSet);
+            }
+        }
+    }
+    //-----------------------------------------------------------------------
+    void Skeleton::_notifyManualBonesDirty(void)
+    {
+        mManualBonesDirty = true;
+    }
+    //-----------------------------------------------------------------------
+    void Skeleton::_notifyManualBoneStateChange(Bone* bone)
+    {
+        if (bone->isManuallyControlled())
+            mManualBones.insert(bone);
+        else
+            mManualBones.erase(bone);
+    }
     //-----------------------------------------------------------------------
     unsigned short Skeleton::getNumBones(void) const
     {
         return (unsigned short)mBoneList.size();
     }
     //-----------------------------------------------------------------------
-    void Skeleton::_getBoneMatrices(Matrix4* pMatrices)
+    void Skeleton::_getBoneMatrices(Affine3* pMatrices)
     {
         // Update derived transforms
         _updateTransforms();
@@ -491,12 +485,12 @@ namespace Ogre {
     //---------------------------------------------------------------------
     Animation* Skeleton::getAnimation(unsigned short index) const
     {
-		// If you hit this assert, then the index is out of bounds.
+        // If you hit this assert, then the index is out of bounds.
         assert( index < mAnimationsList.size() );
 
         AnimationList::const_iterator i = mAnimationsList.begin();
 
-		std::advance(i, index);
+        std::advance(i, index);
 
         return i->second;
     }
@@ -520,11 +514,11 @@ namespace Ogre {
         return i->second;
 
     }
-	//---------------------------------------------------------------------
-	bool Skeleton::hasBone(const String& name) const 
-	{	
-		return mBoneListByName.find(name) != mBoneListByName.end();
-	}
+    //---------------------------------------------------------------------
+    bool Skeleton::hasBone(const String& name) const 
+    {   
+        return mBoneListByName.find(name) != mBoneListByName.end();
+    }
     //---------------------------------------------------------------------
     void Skeleton::deriveRootBone(void) const
     {
@@ -537,12 +531,11 @@ namespace Ogre {
 
         mRootBones.clear();
 
-        Bone* currentBone;
         BoneList::const_iterator i;
         BoneList::const_iterator iend = mBoneList.end();
         for (i = mBoneList.begin(); i != iend; ++i)
         {
-            currentBone = *i;
+            Bone* currentBone = *i;
             if (currentBone->getParent() == 0)
             {
                 // This is a root
@@ -588,17 +581,14 @@ namespace Ogre {
             of << "-- Animation '" << anim->getName() << "' (length " << anim->getLength() << ") --" << std::endl;
             of << "Number of tracks: " << anim->getNumNodeTracks() << std::endl;
 
-            int ti;
-            for (ti = 0; ti < anim->getNumNodeTracks(); ++ti)
+            for (unsigned short ti = 0; ti < anim->getNumNodeTracks(); ++ti)
             {
                 NodeAnimationTrack* track = anim->getNodeTrack(ti);
                 of << "  -- AnimationTrack " << ti << " --" << std::endl;
-                of << "  Affects bone: " << ((Bone*)track->getAssociatedNode())->getHandle() << std::endl;
+                of << "  Affects bone: " << static_cast<Bone*>(track->getAssociatedNode())->getHandle() << std::endl;
                 of << "  Number of keyframes: " << track->getNumKeyFrames() << std::endl;
 
-                int ki;
-                
-                for (ki = 0; ki < track->getNumKeyFrames(); ++ki)
+                for (unsigned short ki = 0; ki < track->getNumKeyFrames(); ++ki)
                 {
                     TransformKeyFrame* key = track->getNodeKeyFrame(ki);
                     of << "    -- KeyFrame " << ki << " --" << std::endl;
@@ -618,15 +608,15 @@ namespace Ogre {
 
     }
     //---------------------------------------------------------------------
-	SkeletonAnimationBlendMode Skeleton::getBlendMode() const
+    SkeletonAnimationBlendMode Skeleton::getBlendMode() const
     {
-		return mBlendState;
-	}
+        return mBlendState;
+    }
     //---------------------------------------------------------------------
-	void Skeleton::setBlendMode(SkeletonAnimationBlendMode state) 
+    void Skeleton::setBlendMode(SkeletonAnimationBlendMode state) 
     {
-		mBlendState = state;
-	}
+        mBlendState = state;
+    }
     //---------------------------------------------------------------------
     Skeleton::BoneIterator Skeleton::getRootBoneIterator(void)
     {
@@ -650,11 +640,11 @@ namespace Ogre {
         {
             (*i)->_update(true, false);
         }
-		mManualBonesDirty = false;
+        mManualBonesDirty = false;
     }
     //---------------------------------------------------------------------
-	void Skeleton::optimiseAllAnimations(bool preservingIdentityNodeTracks)
-	{
+    void Skeleton::optimiseAllAnimations(bool preservingIdentityNodeTracks)
+    {
         AnimationList::iterator ai, aiend;
         aiend = mAnimationsList.end();
 
@@ -685,53 +675,53 @@ namespace Ogre {
         for (ai = mAnimationsList.begin(); ai != aiend; ++ai)
         {
             // Don't discard identity node tracks here
-			ai->second->optimise(false);
-		}
-	}
-	//---------------------------------------------------------------------
-	void Skeleton::addLinkedSkeletonAnimationSource(const String& skelName, 
-		Real scale)
-	{
-		// Check not already linked
-		LinkedSkeletonAnimSourceList::iterator i;
-		for (i = mLinkedSkeletonAnimSourceList.begin(); 
-			i != mLinkedSkeletonAnimSourceList.end(); ++i)
-		{
-			if (skelName == i->skeletonName)
-				return; // don't bother
-		}
+            ai->second->optimise(false);
+        }
+    }
+    //---------------------------------------------------------------------
+    void Skeleton::addLinkedSkeletonAnimationSource(const String& skelName, 
+        Real scale)
+    {
+        // Check not already linked
+        LinkedSkeletonAnimSourceList::iterator i;
+        for (i = mLinkedSkeletonAnimSourceList.begin(); 
+            i != mLinkedSkeletonAnimSourceList.end(); ++i)
+        {
+            if (skelName == i->skeletonName)
+                return; // don't bother
+        }
 
-		if (isLoaded())
-		{
-			// Load immediately
-			SkeletonPtr skelPtr = 
-				SkeletonManager::getSingleton().load(skelName, mGroup);
-			mLinkedSkeletonAnimSourceList.push_back(
-				LinkedSkeletonAnimationSource(skelName, scale, skelPtr));
+        if (isLoaded())
+        {
+            // Load immediately
+            SkeletonPtr skelPtr = static_pointer_cast<Skeleton>(
+                SkeletonManager::getSingleton().load(skelName, mGroup));
+            mLinkedSkeletonAnimSourceList.push_back(
+                LinkedSkeletonAnimationSource(skelName, scale, skelPtr));
 
-		}
-		else
-		{
-			// Load later
-			mLinkedSkeletonAnimSourceList.push_back(
-				LinkedSkeletonAnimationSource(skelName, scale));
-		}
+        }
+        else
+        {
+            // Load later
+            mLinkedSkeletonAnimSourceList.push_back(
+                LinkedSkeletonAnimationSource(skelName, scale));
+        }
 
-	}
-	//---------------------------------------------------------------------
-	void Skeleton::removeAllLinkedSkeletonAnimationSources(void)
-	{
-		mLinkedSkeletonAnimSourceList.clear();
-	}
-	//---------------------------------------------------------------------
-	Skeleton::LinkedSkeletonAnimSourceIterator 
-	Skeleton::getLinkedSkeletonAnimationSourceIterator(void) const
-	{
-		return LinkedSkeletonAnimSourceIterator(
-			mLinkedSkeletonAnimSourceList.begin(), 
-			mLinkedSkeletonAnimSourceList.end());
-	}
-	//---------------------------------------------------------------------
+    }
+    //---------------------------------------------------------------------
+    void Skeleton::removeAllLinkedSkeletonAnimationSources(void)
+    {
+        mLinkedSkeletonAnimSourceList.clear();
+    }
+    //---------------------------------------------------------------------
+    Skeleton::LinkedSkeletonAnimSourceIterator 
+    Skeleton::getLinkedSkeletonAnimationSourceIterator(void) const
+    {
+        return LinkedSkeletonAnimSourceIterator(
+            mLinkedSkeletonAnimSourceList.begin(), 
+            mLinkedSkeletonAnimSourceList.end());
+    }
+    //---------------------------------------------------------------------
     struct DeltaTransform
     {
         Vector3 translate;
@@ -1024,5 +1014,18 @@ namespace Ogre {
         }
     }
 
-}
+    size_t Skeleton::calculateSize(void) const
+    {
+        size_t memSize = 0;
+        memSize += sizeof(SkeletonAnimationBlendMode);
+        memSize += mBoneList.size() * sizeof(Bone);
+        memSize += mRootBones.size() * sizeof(Bone);
+        memSize += mBoneListByName.size() * (sizeof(String) + sizeof(Bone*));
+        memSize += mAnimationsList.size() * (sizeof(String) + sizeof(Animation*));
+        memSize += mManualBones.size() * sizeof(Bone*);
+        memSize += mLinkedSkeletonAnimSourceList.size() * sizeof(LinkedSkeletonAnimationSource);
+        memSize += sizeof(bool);
 
+        return memSize;
+    }
+}
